package cz.zcu.kiv.offscreen.graph.creator;

import cz.zcu.kiv.comav.core.export.utils.BindingInformation;
import cz.zcu.kiv.comav.entmm.*;
import cz.zcu.kiv.offscreen.api.EdgeInterface;
import cz.zcu.kiv.offscreen.api.GraphInterface;
import cz.zcu.kiv.offscreen.api.VertexInterface;
import cz.zcu.kiv.offscreen.graph.EdgeImpl;
import cz.zcu.kiv.offscreen.graph.GraphImpl;
import cz.zcu.kiv.offscreen.graph.VertexImpl;
import java.util.*;
import org.apache.log4j.Logger;

/**
 * This class provides creation graph of the loaded bundle.
 *
 * @author Jindra Pavlíková <jindra.pav2@seznam.cz>
 */
public class GraphMaker {

    private GraphInterface graph;
    private Logger logger = Logger.getLogger(GraphMaker.class);
    private List<Component> componnentList;
    private Map<String, VertexInterface> vertexMap;

    public GraphMaker(List<Component> componnentList) {
        logger.trace("ENTRY");
        this.graph = new GraphImpl();
        this.componnentList = componnentList;
        logger.trace("EXIT");
    }

    /**
     * This method creates the vertices and adds them to the graph.
     */
    private void generateVertices() {
        logger.trace("ENTRY");
        VertexInterface vertex;
        String name = "";
        String symbolicName = "";
        int id = 0;
        Element[] exportedPckg;
        Element[] importedPckg;
        this.vertexMap = new HashMap<String, VertexInterface>();
        for (Component comp : this.componnentList) {
            id++;

            symbolicName = createSymbolicName(comp.getTagSet());
            //System.out.println("vertex: " + id + " - " + symbolicName);
            name = comp.getName();
            vertex = new VertexImpl(id, name, symbolicName);

            exportedPckg = (comp.getTraitSet())[0].getElementSet();
            importedPckg = (comp.getTraitSet())[1].getElementSet();

            vertex.setExportedPackages(getListOfPackagesName(exportedPckg));
            vertex.setImportedPackages(getListOfPackagesName(importedPckg));

            this.vertexMap.put(symbolicName, vertex);
            this.graph.addVertex(symbolicName, vertex);
        }
        logger.trace("EXIT");
    }

    /**
     * This method creates the edges and adds them to the graph.
     */
    private void createEdges() {
        logger.trace("ENTRY");
        EdgeInterface edge;
        int id = 0;
        for (Component comp : this.componnentList) {
            System.out.println("createEdges 1");
            ArrayList<Component> allAlienComponents = new ArrayList<Component>();
            Binding binding[] = comp.getBindings();
            for (int j = 0; j < binding.length; j++) {
                System.out.println("createEdges 2");
                Component alienComp = null;
                if (binding[j] instanceof ElementElementBinding) {
                    System.out.println("createEdges 3");
                    if (binding[j].isProvided() == false) {
                        System.out.println("createEdges 4");
                        BindingInformation alien = BindingInformation.createFromElement(this.componnentList,
                                ((ElementElementBinding) binding[j]).getAlien());
                        alienComp = alien.getComponent();
                    }
                }
                /*
                 * else if (binding[j] instanceof ComponentComponentBinding) {
                 * alienComp = ((ComponentComponentBinding) binding[j])
                 * .getAlien(); } else if (binding[j] instanceof
                 * ElementComponentBinding) { alienComp =
                 * ((ElementComponentBinding) binding[j]) .getAlien(); }
                 */

                // it means it is required
                if (alienComp != null && !comp.equals(alienComp)) {
                    System.out.println("createEdges 5");
                    // filter connections to keep only one per every two
                    // components
                    if (!allAlienComponents.contains(alienComp)) {
                        System.out.println("createEdges 6");
                        allAlienComponents.add(alienComp);
                        id++;

                        edge = new EdgeImpl(id, this.vertexMap.get(createSymbolicName(alienComp.getTagSet())).getSymbolicName(), this.vertexMap.get(createSymbolicName(comp.getTagSet())).getSymbolicName());

                        Element[] exportedPackagesFrom = (alienComp.getTraitSet())[0].getElementSet();
                        Element[] importedPackagesTo = (comp.getTraitSet())[1].getElementSet();

                        edge.setPackageConnection(findPackageConections(exportedPackagesFrom, importedPackagesTo));
                        System.out.println(edge.getId() + ": " + edge.getFrom() + " - " + edge.getTo());
                        this.graph.addEdge(edge);
                    }
                }
            }
        }
        logger.trace("EXIT");
    }

    /**
     * Finds collective package connections.
     * @param exportedPackages - array of exported packages
     * @param importedPackages - array of imported packages
     * @return List of collective package connections.
     */
    private List<String> findPackageConections(Element[] exportedPackages, Element[] importedPackages) {
        logger.trace("ENTRY");
        List<String> packageConnections = new LinkedList<String>();
        String importedPack = "";
        String exportedPack = "";

        for (int i = 0; i < exportedPackages.length; i++) {
            exportedPack = exportedPackages[i].getName();
            for (int k = 0; k < importedPackages.length; k++) {
                importedPack = importedPackages[k].getName();
                if (exportedPack.equals(importedPack)) {
                    packageConnections.add(importedPack);
                }
            }
        }
        logger.trace("EXIT");
        return packageConnections;
    }

    /**
     * Gets list of package names.
     * @param packages - array with packages.
     * @return List of package names.
     */
    private List<String> getListOfPackagesName(Element[] packages) {
        List<String> packagesNames = new LinkedList<String>();

        for (int i = 0; i < packages.length; i++) {
            packagesNames.add(packages[i].getName());
        }
        return packagesNames;
    }

    /**
     * Create symbolic name.
     * @param tag 
     * @return Symbolic name.
     */
    private String createSymbolicName(Tag[] tag) {
        logger.trace("ENTRY");
        logger.trace("EXIT");
        return tag[0].getValue() + " " + tag[1].getValue();
    }

    /**
     * Generates the graph with vertices and edges.
     *
     * @return graph
     */
    public GraphInterface generate() {
        logger.trace("ENTRY");
        this.generateVertices();
        this.createEdges();
        logger.trace("EXIT");
        return this.graph;
    }
}
