package cz.zcu.kiv.offscreen.servlets;

import cz.zcu.kiv.comav.entmm.Component;
import cz.zcu.kiv.offscreen.api.GraphInterface;
import cz.zcu.kiv.offscreen.api.VertexInterface;
import cz.zcu.kiv.offscreen.graph.GraphExport;
import cz.zcu.kiv.offscreen.graph.creator.GraphMaker;
import cz.zcu.kiv.offscreen.graph.loader.GenericComponentLoader;
import cz.zcu.kiv.offscreen.loader.configuration.ConfigurationLoader;
import cz.zcu.kiv.offscreen.session.SessionManager;
import cz.zcu.kiv.offscreen.storage.FileManager;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import java.util.Map;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.sf.json.JSONObject;
import org.apache.log4j.Logger;

/**
 *
 * @author Jindra Pavlíková <jindra.pav2@seznam.cz>
 */
public class LoadGraphData extends HttpServlet {

    private Logger logger = Logger.getLogger(LoadGraphData.class);

    /**
     * Handles the HTTP
     * <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        logger.trace("ENTRY");
        response.setContentType("application/json");
        PrintWriter out = response.getWriter();
        try {
            String sessionValue = SessionManager.getSessionValue(request, "JSESSIONID");
            String storageLocation = ConfigurationLoader.getStorageLocation(request.getServletContext());

            if ( request.getParameter("diagram_hash") != null ) {
            	System.out.println("diagram_hash nastaven");
            	sessionValue =  request.getParameter("diagram_hash");
            }
            
            String file_directory = sessionValue;
            if ( request.getParameter("diagram_id") != null ) {
            	file_directory = sessionValue + "_" + request.getParameter("diagram_id");
            }
            
            FileManager fileManager = new FileManager(file_directory, storageLocation);
            String framework = SessionManager.getSessionValue(request, "framework");
            
            if ( framework.length() == 0 ) {
            	framework = "osgi";
            }
            
            logger.debug("framework: " + framework);
            GenericComponentLoader loader = new GenericComponentLoader(framework);

            loader.load(fileManager.convertFileToURI());
            List<Component> componnentList = loader.getComponents();
            GraphMaker graphMaker = new GraphMaker(componnentList);
            GraphInterface graph = graphMaker.generate();
            Map<String, VertexInterface> map = graph.getVertices();
//            for (String key : map.keySet()) {
//                System.out.println("generate" + map.get(key).getId() + " - " + map.get(key).getSymbolicName());
//            }
            GraphExport export = new GraphExport(graph);
//            System.err.println(export.getEdges().size());
//            for (EdgeInterface edge : export.getEdges()) {
//                System.out.println(edge.getId() + " - " + edge.getFrom() + " - " + edge.getTo());
//            }
            JSONObject o = JSONObject.fromObject(export);

            out.write(o.toString());


        } finally {
            out.close();
        }
        logger.trace("EXIT");
    }

    /**
     * Handles the HTTP
     * <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        logger.trace("ENTRY");
        logger.trace("EXIT");
        throw new UnsupportedOperationException();
    }
}
