package cz.zcu.kiv.offscreen.servlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import cz.zcu.kiv.offscreen.user.DB;
import cz.zcu.kiv.offscreen.user.User;

public class Register extends HttpServlet{
	
    private Logger logger = Logger.getLogger(Register.class); 
    
    private static final String EMAIL_ADDRESS_PATTERN = "^[A-Za-z0-9-\\+_]+(\\.[A-Za-z0-9-_]+)*@" +
    											"[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*" +
    											"(\\.[A-Za-z]{2,})$";
    
    /**
     * Handles the HTTP
     * <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        logger.trace("ENTRY");
        RequestDispatcher rd = getServletContext().getRequestDispatcher("/uploadComponent.jsp");
        
        rd.forward(request, response);
        logger.trace("EXIT");
    }
    
    /**
     * Handles the HTTP
     * <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        logger.trace("ENTRY");
        //RequestDispatcher rd = getServletContext().getRequestDispatcher("/uploadComponent.jsp");

        response.setContentType("application/json");
        PrintWriter out = response.getWriter();
        
        
    	DB db_conn = new DB(getServletContext());
    	
    	User tmpUser = new User (db_conn);
    	
    	String err = "";
    	if ( request.getParameter("user_name") == null || request.getParameter("user_name").length() == 0 ) err += "Please fill Name.";
    	if ( request.getParameter("user_email") == null || request.getParameter("user_email").length() == 0 ){
    		err += "Please fill E-mail.";
    	}else if (tmpUser.existsEmail(request.getParameter("user_email"))){
    		err += "E-mail already exists.";
    	}else if (!isEmailAddressValid(request.getParameter("user_email"))){
    		err += "Please correct e-mail address.";
    	}
    	if ( request.getParameter("user_nick") == null || request.getParameter("user_nick").length() == 0 ){
    		err += "Please fill Login name.";
    	}else if (tmpUser.existsNick(request.getParameter("user_nick"))){
    		err += "Login name already exists.";    		
    	}
    	
    	if ( request.getParameter("user_password") == null || request.getParameter("user_password").length() == 0 ||
    	     request.getParameter("user_password_2") == null || request.getParameter("user_password_2").length() == 0 ){
    		err += "Please fill Password.";
    	} else if ( request.getParameter("user_password").compareTo(request.getParameter("user_password_2")) != 0) {
    		err += "Passwords must be same.";    		
    	} else if ( request.getParameter("user_password").length() <=4 ){
    		err += "Passwords must be at least 5 characters long.";
    	}
    	
    	
    	
    	if (err.length() == 0 ){
    		HashMap<String, String> user_param = new HashMap<String, String>();
    		user_param.put("name", request.getParameter("user_name"));
    		user_param.put("email", request.getParameter("user_email"));
    		user_param.put("nick", request.getParameter("user_nick"));
    		user_param.put("password", request.getParameter("user_password"));
    		user_param.put("session", request.getSession().getId());
	        
    		tmpUser.register(user_param);
    		
    		
    		out.write("{\"ok\":\"ok\"}");
    	}else{
    		out.write("{\"err\":\""+ err +"\"}");    		
    	}
    	
    	out.close();
    	
    	/*
    	
    	if ( logged_user_c.login(request.getParameter("login_name"),request.getParameter("password")) ) {
	    	
	    	request.getSession().setAttribute("logged_user", "1");
	    	request.getSession().setAttribute("logged_user_id", logged_user_c.getId() );
	    	request.getSession().setAttribute("logged_user_nick", logged_user_c.getNick());
    	}else{
    		request.getSession().setAttribute("logged_user", "0");
	    	request.getSession().setAttribute("logged_user_nick", "");        		
    		
    	}*/
        
        
        //presmerovani na stranku s nahravanim komponent
        //response.sendRedirect(getServletContext().getInitParameter("HOME_URL") + "UploadFiles");

        logger.trace("EXIT");
    }
    
    /**
     * Method verify validity of email address
     * 
     * @param emailAddr - email address
     * @return true - email address is ok
     * 		   false - email address is not correct
     */
    private boolean isEmailAddressValid(String emailAddr){
    	Pattern pattern = Pattern.compile(EMAIL_ADDRESS_PATTERN);
    	Matcher matcher = pattern.matcher(emailAddr);
		return matcher.matches();
    }
	
}
