
/**
 * This class represents creating and sorting of marks.
 */    
var GridMarks = new Class({
    CONST_COUNT_MARKS_ON_ROW: 5,
    CONST_WIDTH_MARK: 21,
    CONST_HEIGHT_MARK: 21,
    
    marks: null,
    position: {
        rx: -60, 
        ry: 13, 
        tx: -50, 
        ty: 23
    },
    
    /**
     * Constructor.
     */
    initialize: function(){
        this.marks = new Hash();
    },
    
    /**
     * Add mark to list.
     * Parameters: markId - id of added mark
     *             mark - mark with symbol and color
     */
    addMark: function(markId, mark){
        //console.info(">>>>>" + mark.symbol + "<<<<<");
        if(mark.symbol != "undefined" || mark.symbol != "" || mark.symbol != undefined){
            var translate = this.countTranslate(this.marks.getLength());
            var markGroup = this.createMarkSvg(mark, this.position, translate);
            this.marks.add(markId,{
                mark: mark, 
                $markGroup: $(markGroup)
            });
        
            mark.$vertexSelector.append(markGroup);
        }
        
        
    },
    
    /**
     * Count translate of mark in grids.
     */
    countTranslate: function(length){
        var x = (length % this.CONST_COUNT_MARKS_ON_ROW)*this.CONST_WIDTH_MARK;
        var y = Math.floor(length / this.CONST_COUNT_MARKS_ON_ROW)*this.CONST_HEIGHT_MARK;
        
        return {
            x: x, 
            y: y
        };
    },
    
    /**
     * Removes mark with given id from list.
     * Parameters: markId - mark id
     */
    removeMark: function(markId){
        var item = this.marks.remove(markId).$markGroup;
        item.remove();
        this.reorganizeMarks();
    },
    
    /**
     * This methor organized marks in grid.
     */
    reorganizeMarks: function(){
        var countMark = 0;
        for (var key in this.marks.array){
            if(this.marks.array.hasOwnProperty(key)){
               // console.info(this.marks.array[key].mark.id);
                this.marks.array[key].$markGroup.remove();
                var translate = this.countTranslate(countMark);
               // console.info(translate.x + " - "+ translate.y);
                var markGroup = this.createMarkSvg(this.marks.array[key].mark, this.position, translate);
                this.marks.array[key].$markGroup = $(markGroup);
                this.marks.array[key].mark.$vertexSelector.append(markGroup);
                    
                countMark++;
            }
        }
    },
    
    /**
     * Create svg element which represents mark in graph.
     */
    createMarkSvg: function(mark, position, translate){
        //console.info(translate.x + " - " + translate.y);
        var group = createSvgElement('g',{
            "class": mark.id,
            transform: 'translate(' + (translate.x) + ',' + translate.y + ')'
        });
        var rect= createSvgElement('rect', {
            rx: 4, 
            ry: 4, 
            height: this.CONST_HEIGHT_MARK, 
            width: this.CONST_WIDTH_MARK, 
            //            x: position['rx'], 
            //            y: position['ry'], 
            y: 2*position['ry'],
            stroke: 'black', 
            fill: mark.symbol[1]
        });
        var label = createSvgElement('text', {
            x: 10, 
            y: position['ty']+13, 
            fill: 'black',
            'text-anchor': 'middle',
            'dominant-baseline': 'baseline'
        });
        
        var myText = document.createTextNode(mark.symbol[0]);
        label.appendChild(myText);
        group.appendChild(rect);
        group.appendChild(label);  
        
        return group;
    },
    
    /**
     * Tests if mark with given id is in list of mark.
     * Parameters: markId - mark id
     * If mark exists, it is returned
     */
    existMark: function(markId){
        return this.marks.contains(markId);
    }
});
