/**
 * Configuration of detail vertex tooltip (setting style).
 */
function configurationVertexDetailTooltip(selectorString){
    $(selectorString).each(function(){
        $(this).qtip({
            show: {
                solo: true,
                delay: 500
            },
            hide: {
                event: 'click'
            },
            content: {
                text: getTooltipVertex(stringToInt($(this).attr("data-vertexId"))),
                title: {
                    text: getSymbolicName(stringToInt($(this).attr("data-vertexId"))),
                    button: 'Close' 
                }
            },
            position: {
                my: "top right",
                at: "bottom right",
                adjust: {
                    y: 2
                }
            //,
            // viewport: $("#viewport")
            },
            style: {
                tip: {
                    corner: 'top right'
                },
                classes: "ui-tooltip-green ui-tooltip-rounded ui-tooltip-shadow"
                            
            }
        });
    });
}

/**
 * Configuration of vertex tooltip (setting style).
 */
function configurationVertexTooltip(selectorString){
    $(selectorString).each(function(){
        $(this).qtip({
            show: {
                solo: true,
                delay: 0,
                event: 'click'
            },
            content: {
                text: getTooltipVertex(stringToInt($(this).attr("data-vertexId"))),
                title: {
                    text: getSymbolicName(stringToInt($(this).attr("data-vertexId")))
                }
            },
            position: {
                my: "top center",
                at: "bottom center",
                adjust: {
                    x: 2
                }
            },
            style: {
                tip: {
                    corner: 'top center'
                },
                classes: "ui-tooltip-green ui-tooltip-rounded ui-tooltip-shadow"
                            
            }
        });
    });
}

/**
 * Configuration of edge tooltip (setting style).
 */
function configurationEdgeTooltip(selectorString){
    $(selectorString).each(function(){
        $(this).qtip({
            show: {
                solo: true,
                delay: 0,
                event: 'click'
            },
            hide: {
                delay: 750,
                events: "mousemove"
            },
            content: {
                text: getTooltipEdge(stringToInt($(this).attr("data-edgeId"))),
                title: {
                    text: 'Involved packages'
                }
            },
            position: {
                target: "mouse",
                adjust: {
                    x: 20,
                    mouse: false
                },
                viewport: $("#viewport")
            },
            style: {
                classes: "ui-tooltip-green ui-tooltip-rounded ui-tooltip-shadow"
                            
            }
        });
    });
}

/**
 * Return tooltip fo vertex wiht given id.
 * parameters: id - id of vertex
 */
function getTooltipVertex(id){
    var exportedPackages = getExportedPackages(id);
    var importedPackages = getImportedPackages(id);
    var tooltip = "<div><ul class='tooltip_vertex_left'>";
    tooltip += "<li class='tooltip_import_package'>" + "Import packages:" + "</li>";
    for(var i = 0; i < importedPackages.length; i++){
        tooltip += "<li>" + importedPackages[i] + "</li>";
    }
    tooltip += "</ul><ul class='tooltip_vertex_right'>";
    tooltip += "<li class='tooltip_export_package'>" + "Export packages:" + "</li>";
    for(var j = 0; j < exportedPackages.length; j++){
        tooltip += "<li>" + exportedPackages[j] + "</li>";
    }
    tooltip += "</ul><br class='clean'/></div>";
    return tooltip;
}

/**
 * Set tooltip to edge.
 * Parameters: id - id of edge
 * Returns tooltip of edge with given id.
 */
function getTooltipEdge(id){
    var connectionList = getConnections(id);
    var tooltip = "<div><ul>";
    if(connectionList.length == 0){
        tooltip += "<li>No packages</li>";
    }
    for(var i = 0; i < connectionList.length; i++){
        tooltip += "<li>" + connectionList[i] + "</li>";
    }
    tooltip += "</ul></div>";
    return tooltip;
}

/**
 * Return symbolic name of vertex with given id.
 * parameters: idVertex - id of vertex
 */
function getSymbolicName(idVertex){
    return GraphManager.graph.vertices[idVertex-1].symbolicName;
}
            
/**
 * Return exported packages one vertex.
 * parameters: idVertex - id of vertex for which wil be return exported packages
 */            
function getExportedPackages(idVertex){
    return GraphManager.graph.vertices[(idVertex-1)].exportedPackages;
}

/**
 * Return imported packages one vertex.
 * parameters: idVertex - id of vertex for which wil be return imported packages
 */ 
function getImportedPackages(idVertex){
    return GraphManager.graph.vertices[(idVertex-1)].importedPackages;
}

/**
 * Return package that are connected two vertices.
 * parameters: idEdge - id of edge
 */ 
function getConnections(idEdge){
    return GraphManager.graph.edges[(idEdge-1)].packageConnections;
}
