package cz.zcu.kiv.comav.loaders.sofa2.wizard;

import java.net.URI;

import org.eclipse.jface.wizard.Wizard;

import cz.zcu.kiv.comav.core.export.loaders.ComponentLoader;

public class LoaderWizard extends Wizard {

	private String projectName;
	private String projectLocation;
	private ComponentLoader loader;
	private URI assembly;

	private ProjectNamePage projectPage;
	private SelectLocalProjectPage localPage;
	private SelectRemoteProjectPage remotePage;
	private SelectAssemblyPage assemblyPage;

	public LoaderWizard() {
		super();
		setNeedsProgressMonitor(true);
	}

	@Override
	public void addPages() {
		projectPage = new ProjectNamePage();
		localPage = new SelectLocalProjectPage();
		assemblyPage = new SelectAssemblyPage();
		remotePage = new SelectRemoteProjectPage();
		addPage(projectPage);
		addPage(localPage);
		addPage(remotePage);
		addPage(assemblyPage);
	}
	
	@Override
	public boolean canFinish() {
		if ((!projectPage.getRemote() && projectPage.isPageComplete() && localPage.isPageComplete() && assemblyPage.isPageComplete())
				|| (projectPage.getRemote() && projectPage.isPageComplete() && remotePage.isPageComplete() && assemblyPage.isPageComplete()))
			return true;
		else 
			return false;
	}

	@Override
	public boolean performFinish() {
		projectName = projectPage.getProjectName();
		if (projectPage.getRemote()) {
			projectLocation = remotePage.getLocation();
		}
		else {
			projectLocation = localPage.getProjectLocation();
		}
		loader = assemblyPage.getComponentLoader();
		assembly = assemblyPage.getSelectedAssembly();

		return true;
	}
	
	public boolean getRemote() {
		return projectPage.getRemote();		
	}

	public String getProjectName() {
		return projectName;
	}

	public String getProjectLocation() {

		return projectLocation;
	}

	public SelectLocalProjectPage getLocalPage() {
		return localPage;
	}

	public SelectRemoteProjectPage getRemotePage() {
		return remotePage;
	}

	public SelectAssemblyPage getAssemblyPage() {
		return assemblyPage;
	}

	public ComponentLoader getLoader() {
		return loader;
	}

	public URI getAssembly() {
		return assembly;
	}

	
}