package cz.zcu.kiv.comav.loaders.sofa2.wizard;

import java.io.File;
import java.net.URI;
import java.util.ArrayList;

import org.eclipse.emf.common.util.EList;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.objectweb.dsrg.sofa.repository.ConfigurationHelper;
import org.objectweb.dsrg.sofa.repository.RepositoryAgent;
import org.objectweb.dsrg.sofa.repository.RepositoryQuery;
import org.objectweb.dsrg.sofa.repository.model.AssemblyDescriptor;
import org.objectweb.dsrg.sofa.repository.model.RepositoryData;

import cz.zcu.kiv.comav.core.export.loaders.ComponentLoader;
import cz.zcu.kiv.comav.loaders.sofa2.Sofa2LocalComponentLoader;
import cz.zcu.kiv.comav.loaders.sofa2.Sofa2RepositoryComponentLoader;

public class SelectAssemblyPage extends WizardPage {
	private Composite container;
	private Combo assemblies;
	private ComponentLoader loader;

	protected SelectAssemblyPage() {
		super("Select assembly");
		setTitle("Select assembly");
		setDescription("Please select assembly, which will be analyzed for visualization.");
	}
	
	private WizardPage getThis() {
		return this;
	}

	@Override
	public void createControl(Composite parent) {
		container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 2;
		
		final Button button = new Button(container, SWT.NONE);
		button.setText("Load assemblies");
		button.addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {
				try {
					getThis().setErrorMessage(null);
					assemblies.removeAll();
					ArrayList<URI> loadedAssemblies = new ArrayList<URI>();
					
					boolean isRemote = ((LoaderWizard)getWizard()).getRemote();
					SelectLocalProjectPage localPage = ((LoaderWizard)getWizard()).getLocalPage();
					SelectRemoteProjectPage remotePage = ((LoaderWizard)getWizard()).getRemotePage();
					if (!isRemote) {
						String location = localPage.getProjectLocation();
						Sofa2LocalComponentLoader tempLoader = new Sofa2LocalComponentLoader(new File(location));
						loadedAssemblies.addAll(tempLoader.getAssemblies());
						loader = tempLoader;
						if (loadedAssemblies.size() == 0)
							getThis().setMessage("There was no assembly in the given path, please return to the previous page.");
						else 
							getThis().setMessage("Assemblies were loaded succesfully.");
					}
					else {
						String location = remotePage.getLocation();
						loader = new Sofa2RepositoryComponentLoader(location);
						RepositoryAgent agent= new RepositoryAgent(ConfigurationHelper.createStandardConfiguration(location));
						RepositoryQuery query = agent.getQuery();
						RepositoryData data = query.getRepositoryDataObject();
						if (data != null) {
							EList<AssemblyDescriptor> assemblyDescriptors = data.getAssemblyDescriptor();
							for (AssemblyDescriptor desc : assemblyDescriptors) {
								loadedAssemblies.add(URI.create(desc.getName() + "?version=" + desc.getVersion()));
							}
						}
					}
					for (URI assembly : loadedAssemblies)
						assemblies.add(assembly.toString());
				} catch (Exception exc) {
					getThis().setErrorMessage("The given location was invalid, please return to the previous page.");
				}
			}

			@Override
			public void mouseDown(MouseEvent e) {
				// noop
			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {
				// noop
			}
		});
		Label infoLabel = new Label(container, SWT.NULL);
		infoLabel.setText("");

		// First row
		Label projectNameLabel = new Label(container, SWT.NULL);
		projectNameLabel.setText("Select from this list");
		
		assemblies = new Combo(container, SWT.READ_ONLY);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		assemblies.setLayoutData(gd);
		assemblies.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				setPageComplete(true);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {		
			}
			
		});
		
		setControl(container);
		setPageComplete(false);
	}

	public ComponentLoader getComponentLoader() {
		return loader;
	}
	
	public URI getSelectedAssembly() {
		int selectedItemIndex = assemblies.getSelectionIndex();
		if (selectedItemIndex != -1) {
			return URI.create(assemblies.getItem(selectedItemIndex));
		}
		else
			return null;
	}
}
