package cz.zcu.kiv.comav.loaders.sofa2.wizard;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.objectweb.dsrg.sofa.repository.ConfigurationHelper;
import org.objectweb.dsrg.sofa.repository.RepositoryAgent;
import org.objectweb.dsrg.sofa.repository.RepositoryQuery;
import org.objectweb.dsrg.sofa.repository.model.RepositoryData;

public class SelectRemoteProjectPage extends WizardPage {
	private Composite container;
	private Text hostTextField;
	private Text pathTextField;
	private Spinner portSpinner;
	private Label connectionLabel;
	
	protected SelectRemoteProjectPage() {
		super("Choose project source");
		setTitle("Choose remote repository");
		setDescription("You can choose any SOFA 2 repository.");
	}

	@Override
	public void createControl(Composite parent) {
		container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 2;

		// First row
		Label infoLabel = new Label(container, SWT.NULL);
		infoLabel.setText("Host");
		hostTextField = new Text(container, SWT.BORDER | SWT.SINGLE);
		hostTextField.setText("localhost");
		hostTextField.addKeyListener(new KeyListener() {

			@Override
			public void keyPressed(KeyEvent e) {
				// noop
			}

			@Override
			public void keyReleased(KeyEvent e) {
				if (!hostTextField.getText().isEmpty() && !pathTextField.getText().isEmpty()) {
					setPageComplete(true);
				}
				else
					setPageComplete(false);
			}

		});
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		hostTextField.setLayoutData(gd);
		
		//second row
		infoLabel = new Label(container, SWT.NULL);
		infoLabel.setText("Path");
		pathTextField = new Text(container, SWT.BORDER | SWT.SINGLE);
		pathTextField.setText("/SofaServlet");
		pathTextField.addKeyListener(new KeyListener() {

			@Override
			public void keyPressed(KeyEvent e) {
				// noop
			}

			@Override
			public void keyReleased(KeyEvent e) {
				if (!hostTextField.getText().isEmpty() && !pathTextField.getText().isEmpty()) {
					setPageComplete(true);
				}
				else
					setPageComplete(false);
			}

		});
		gd = new GridData(GridData.FILL_HORIZONTAL);
		pathTextField.setLayoutData(gd);
		
		// third row
		infoLabel = new Label(container, SWT.NULL);
		infoLabel.setText("Port");
		portSpinner= new Spinner (container, SWT.BORDER);
		portSpinner.setMinimum(1000);
		portSpinner.setMaximum(65535);
		portSpinner.setSelection(8173);
		portSpinner.setIncrement(1);	
		
		final Button button = new Button(container, SWT.NONE);
		button.setText("Test connection");
		button.addMouseListener(new MouseListener() {

			@Override
			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void mouseUp(MouseEvent e) {
				RepositoryAgent agent;
				try {
					agent = new RepositoryAgent(ConfigurationHelper.createStandardConfiguration(getLocation()));
					RepositoryQuery query = agent.getQuery();
					RepositoryData data = query.getRepositoryDataObject();
					if (data != null)
						connectionLabel.setText("Repository path verified.");
					else
						connectionLabel.setText("Connection to repository failed.");
				} catch (MalformedURLException e1) {
					connectionLabel.setText("Connection to repository failed.");
				}
			}
			
		});
		connectionLabel = new Label(container, SWT.NULL);
		connectionLabel.setText("Connection was not tested yet...");
		
		setControl(container);
		setPageComplete(true);
	}
	
	public String getLocation() {
		try {
			URL url = new URL("http", hostTextField.getText(), portSpinner.getSelection(), pathTextField.getText());
			return url.toString();
		} catch (MalformedURLException ex) {
			return null;
		}
	}
}
